	program test_fpe_handler
C--- Demonstrate IEEE floating point arithmetic exception handling under the
C--- Linux/Intel i386 architecture.  The six floating point exception
C--- conditions are tested individually with and without exception handling
C--- enabled.
C--- T.Crane, March 2003.
 	implicit double precision (a-h,o-z)
C--- Define the i387 floating point unit's interrupt mask values
	parameter (I_FPU_MASK_IM=1)	! Invalid operation mask
	parameter (I_FPU_MASK_DM=2)	! Denormalised operand mask
	parameter (I_FPU_MASK_ZM=4)	! Zero-divide mask
	parameter (I_FPU_MASK_OM=8)	! Overflow mask
	parameter (I_FPU_MASK_UM=16)	! Underflow mask
	parameter (I_FPU_MASK_PM=32)	! Precision (inexact result) mask

	print *,'Enter test number ?'
	print *,'Test 1 - floating point overflow'
	print *,'Test 2 - invalid operation'
	print *,'Test 3 - zero-divide'
	print *,'Test 4 - denormalised result'
	print *,'Test 5 - floating point underflow'
	print *,'Test 6 - inexact result'

	read (5,*) itest
C--- Test 1, floating point overflow
	if (itest .eq. 1) then
	 a=1d300
	 icnt=0
	 do while (icnt .lt. 12)
	  print *,'a=',a
	  a=a*10
	 icnt=icnt+1
	 enddo
	 print *,'Now enable overflow interrupt' 
         call setup_fpe_handler(I_FPU_MASK_OM)
	 a=1d300
	 do while (.true.)
	  print *,'a=',a
	  a=a*10
	 enddo
C---Test 2, invalid operation 
	else if (itest .eq. 2) then
	 a=-2
	 b=sqrt(a)
	 print *,'sqrt(-2)=',b
	 print *,'Now enable invalid operations interrupt'
         call setup_fpe_handler(I_FPU_MASK_IM)
	 b=sqrt(a)
	 print *,'sqrt(-2)=',b
C--- Test 3, division by zero
	else if (itest .eq. 3) then
	 a=0
	 b=1/a
	 print *,'1/0=',b
	 print *,'Now enable zero-divide interrupt'
         call setup_fpe_handler(I_FPU_MASK_ZM)
	 b=1/a
	 print *,'1/0=',b
C--- Test 4, denormalised result
	else if (itest .eq. 4) then
	 a=  2.3D-308
	 print *,'a=2.3D-308=',a
	 a=  2.2D-308
	 print *,'a=2.2D-308=',a
	 print *,'Now enable denormalisation interrupt'
         call setup_fpe_handler(I_FPU_MASK_DM)
	 a=  2.3D-308
	 print *,'a=2.3D-308=',a
	 a=  2.2D-308
	 print *,'a=2.2D-308=',a
C--- Test 5, floating point underflow
	else if (itest .eq. 5) then
	 a=1d-300
	 do while (a .gt. 0d0)
	  a=a/2
	  print *,'a=',a
	 enddo
	 print *,'Now enable underflow interrupt'
         call setup_fpe_handler(I_FPU_MASK_UM)
	 a=1d-300
	 do while (a .gt. 0d0)
	  a=a/2
	  print *,'a=',a
	 enddo
C--- Test 6, inexact result
	else if (itest .eq. 6) then
	 a=1
	 b=2
	 c=10
	 x=a/b		! This division produces an exact result
	 print *,'x=',x
	 y=a/c		! This division produces an inexact result
	 print *,'y=',y
	 print *,'Now enable inexact result interrupt'
         call setup_fpe_handler(I_FPU_MASK_PM)
	 x=a/b		! This division produces an exact result
	 print *,'x=',x
	 y=a/c		! This division produces an inexact result
	 print *,'y=',y
	endif
	stop
	end
